# -*- coding: utf-8 -*-
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface
from ..exceptions import *
from ..utils import *

class Profile(PlivoResource):
    _name = 'Profile'
    _identifier_string = 'Profile_uuid'

class Profile(PlivoResourceInterface):
    _resource_type = Profile

    @validate_args(profile_uuid=[of_type(six.text_type)])
    def get(self, profile_uuid):
        return self.client.request(
            'GET', ('Profile', profile_uuid), response_type=None)

    @validate_args(
        limit=[optional(of_type(*six.integer_types))],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list(self, limit=None, offset=None):
        params ={}
        if limit:
            params['limit']= limit
        if offset:
            params['offset'] = offset

        return self.client.request(
            'GET', ('Profile', ), params,
            response_type=None,
            objects_type=None)

    @validate_args(profile_uuid=[of_type(six.text_type)])
    def delete(self, profile_uuid):
        return self.client.request(
            'DELETE', ('Profile', profile_uuid),
            response_type=None,
            objects_type=None)
    
    @validate_args(
        profile_alias=[optional(of_type(six.text_type))],
        customer_type=[required(of_type(six.text_type))],
        entity_type=[required(of_type(six.text_type))],
        company_name=[required(of_type(six.text_type))],
        ein=[optional(of_type(six.text_type))],
        ein_issuing_country=[optional(of_type(six.text_type))],
        stock_symbol=[optional(of_type(six.text_type))],
        stock_exchange=[optional(of_type(six.text_type))],
        website=[optional(of_type(six.text_type))],
        vertical=[required(of_type(six.text_type))],
        alt_business_id=[optional(of_type(six.text_type))],
        alt_business_id_type=[optional(of_type(six.text_type))],
        plivo_subaccount=[optional(of_type(six.text_type))],
        address=[optional(of_type_exact(dict))],
        authorized_contact=[optional(of_type_exact(dict))],
        business_contact_email=[optional(of_type(six.text_type))])
    def create(self,
               profile_alias,
               customer_type,
               entity_type,
               company_name,
               vertical,
               alt_business_id='',
               alt_business_id_type= '',
               plivo_subaccount='',
               ein='',
               ein_issuing_country='',
               stock_symbol='',
               stock_exchange='',
               website='',
               address={},
               authorized_contact={},
               business_contact_email=''):
        return self.client.request('POST', ('Profile', ),
                                   to_param_dict(self.create, locals()))


    # params values should be dictionary like 
    # {'address': {}, 'authorized_contact': {}, 'entity_type':'', 'vertical': '', 'company_name': '', 'website':'', 'business_contact_email':''} 
    def update(self,profile_uuid, params=None):
        if params == None:
            raise ValidationError(
                'required atleast one of powerpack attributes'
            )
        return self.client.request('POST', ('Profile', profile_uuid), params)



# -*- coding: utf-8 -*-
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface
from ..exceptions import *
from ..utils import *

class Brand(PlivoResource):
    _name = 'Brand'
    _identifier_string = 'brand_id'

class Brand(PlivoResourceInterface):
    _resource_type = Brand

    @validate_args(brand_id=[of_type(six.text_type)])
    def get(self, brand_id):
        return self.client.request(
            'GET', ('10dlc','Brand', brand_id), response_type=None)
            
    @validate_args(
        type=[optional(of_type(six.text_type))],
        status=[optional(of_type(six.text_type))],
        limit=[optional(of_type(*six.integer_types))],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list(self, type=None, status=None, 
                limit=None, offset=None):
        return self.client.request(
            'GET', ('10dlc', 'Brand', ),
            to_param_dict(self.list, locals()),
            response_type=None,
            objects_type=None)
    
    @validate_args(
        brand_alias=[required(of_type(six.text_type))],
        brand_type=[optional(of_type(six.text_type), is_in(('STANDARD','STARTER')))],
        profile_uuid=[required(of_type(six.text_type))],
        secondary_vetting=[optional(of_type_exact(bool))],
        url=[optional(of_type(six.text_type))],
        method=[optional(of_type(six.text_type))])
    def create(self,
               brand_alias,
               brand_type,
               profile_uuid,
               secondary_vetting=False,
               url='',
               method='POST'
        ):
        return self.client.request('POST', ('10dlc', 'Brand'),
                                   to_param_dict(self.create, locals()))


    @validate_args(brand_id=[required(of_type(six.text_type))])
    def get_usecases(self, brand_id):
        return self.client.request(
            'GET', ('10dlc','Brand', brand_id, 'usecases'), response_type=None)

    @validate_args(brand_id=[required(of_type(six.text_type))])
    def delete(self, brand_id):
        return self.client.request(
            'DELETE', ('10dlc','Brand', brand_id), response_type=None)



# -*- coding: utf-8 -*-
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface
from ..exceptions import *
from ..utils import *


class Media(PlivoResource):
    _name = 'Media'
    _identifier_string = 'media_id'

    def get(self):
        return self.client.medias.get(self.id)


class Media(PlivoResourceInterface):
    _resource_type = Media

    @validate_args(
        media_file=[optional(of_type_exact(list))])
    def upload(self, media_file):
        if media_file:
            fileList = []
            for media_url in media_file:
                file_extension = media_url.strip().split('.')[-1].lower()
                if file_extension not in ['jpeg', 'jpg', 'png', 'gif', 'mp3', 'mp4', 'mpeg', 'wav', 'ogg', '3gpp', '3gpp2', 'vcard', 'csv', 'pdf', 'xls', 'xlsx', 'xcf', 'plain']:
                    raise ValidationError(
                        'File format of the file to be uploaded should be one of JPG, JPEG, PNG, GIF, MP3, MP4, MPEG, WAV, OGG, 3GPP, 3GPP2, VCARD, CSV, PDF, XLS, XLSX, XCF, or TEXT'
                    )
                content_types = {
                    'jpeg': 'image/jpeg',
                    'jpg': 'image/jpeg',
                    'png': 'image/png',
                    'gif': 'image/gif',
                    'mp3': 'audio/mp3',
                    'mp4': 'video/mp4',
                    'mpeg': 'audio/mpeg',
                    'wav': 'audio/wav',
                    'ogg': 'audio/ogg',
                    '3gpp': 'video/3gpp',
                    '3gpp2': 'video/3gpp2',
                    'vcard': 'text/vcard',
                    'csv': 'text/csv',
                    'pdf': 'application/pdf',
                    'xls': 'application/vnd.ms-excel',
                    'xlsx': 'application/vnd.ms-excel',
                    'xcf': 'image/xcf',
                    'plain': 'text/plain'
                }
                print(media_url)
                import os
                files = (
                    'file', (media_url.split(os.sep)[-1], open(
                        media_url, 'rb'), content_types[file_extension])
                )
                fileList.append(files)
        data_to_send = {}
        return self.client.request(
            'POST', ('Media', ), data_to_send, files=fileList)

    @validate_args(media_id=[of_type(six.text_type)])
    def get(self, media_id):
        return self.client.request(
            'GET', ('Media', media_id), response_type=None)

    @validate_args(
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list(self,
             limit=20,
             offset=0):
        return self.client.request(
            'GET', ('Media', ),
            to_param_dict(self.list, locals()),
            response_type=None)



from plivo import exceptions
from plivo.utils.validators import *
from plivo.xml import PlivoXMLElement, map_type
import six


class StreamElement(PlivoXMLElement):
    _name = 'Stream'
    _nestable = []

    ALLOWED_AUDIO_TRACKS = ['inbound', 'outbound', 'both']
    ALLOWED_CALLBACK_METHODS = ['GET', 'POST']

    @property
    def bidirectional(self):
        return self.__bidirectional

    @bidirectional.setter
    def bidirectional(self, value):
        if value is not None:
            if not isinstance(value, bool):
                raise exceptions.ValidationError("bidirectional must be a boolean value.")
        self.__bidirectional = value

    @validate_args(
        value=[of_type(bool)]
    )
    def set_bidirectional(self, value):
        self.bidirectional = value
        return self

    @property
    def audioTrack(self):
        return self.__audioTrack

    @audioTrack.setter
    def audioTrack(self, value):
        if value is not None:
            if value not in self.ALLOWED_AUDIO_TRACKS:
                raise exceptions.ValidationError(f"audioTrack must be one of {self.ALLOWED_AUDIO_TRACKS}.")
        self.__audioTrack = value

    @validate_args(
        value=[of_type(str)]
    )
    def set_audioTrack(self, value):
        self.audioTrack = value
        return self

    @property
    def statusCallbackMethod(self):
        return self.__statusCallbackMethod

    @statusCallbackMethod.setter
    def statusCallbackMethod(self, value):
        if value is not None:
            if value not in self.ALLOWED_CALLBACK_METHODS:
                raise exceptions.ValidationError(f"statusCallbackMethod must be one of {self.ALLOWED_CALLBACK_METHODS}.")
        self.__statusCallbackMethod = value

    @validate_args(
        value=[of_type(str)]
    )
    def set_statusCallbackMethod(self, value):
        self.statusCallbackMethod = value
        return self

    @property
    def keepCallAlive(self):
        return self.__keepCallAlive

    @keepCallAlive.setter
    def keepCallAlive(self, value):
        if value is not None:
            if not isinstance(value, bool):
                raise exceptions.ValidationError("keepCallAlive must be a boolean value.")
        self.__keepCallAlive = value

    @validate_args(
        value=[of_type(bool)]
    )
    def set_keepCallAlive(self, value):
        self.keepCallAlive = value
        return self

    def __init__(
            self,
            content,
            bidirectional=None,
            audioTrack=None,
            streamTimeout=None,
            statusCallbackUrl=None,
            statusCallbackMethod=None,
            contentType=None,
            extraHeaders=None,
            keepCallAlive=None
    ):
        super(StreamElement, self).__init__()

        self.content = content
        self.bidirectional = bidirectional
        self.audioTrack = audioTrack
        self.streamTimeout = streamTimeout
        self.statusCallbackUrl = statusCallbackUrl
        self.statusCallbackMethod = statusCallbackMethod
        self.contentType = contentType
        self.extraHeaders = extraHeaders
        self.keepCallAlive = keepCallAlive

    def to_dict(self):
        d = {
            'bidirectional': self.bidirectional,
            'audioTrack': self.audioTrack,
            'streamTimeout': self.streamTimeout,
            'statusCallbackUrl': self.statusCallbackUrl,
            'statusCallbackMethod': self.statusCallbackMethod,
            'contentType': self.contentType,
            'extraHeaders': self.extraHeaders,
            'keepCallAlive': self.keepCallAlive
        }
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }


from plivo.utils.validators import *
from plivo.xml import PlivoXMLElement, map_type


class PlayElement(PlivoXMLElement):
    _name = 'Play'
    _nestable = []

    @property
    def loop(self):
        return self.__loop

    @loop.setter
    def loop(self, value):
        self.__loop = int(value) if value is not None else None

    @validate_args(
        value=[of_type(*six.integer_types)],
    )
    def set_loop(self, value):
        self.loop = value
        return self

    def __init__(
            self,
            content,
            loop=None,
    ):
        super(PlayElement, self).__init__()

        self.content = content
        self.loop = loop

    def to_dict(self):
        d = {
            'loop': self.loop,
        }
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }



# -*- coding: utf-8 -*-
"""
Application class - along with its list class
"""

from plivo.base import (ListResponseObject, PlivoResource, PlivoResourceInterface, SecondaryPlivoResource)
from plivo.utils.validators import *
from plivo.utils import to_param_dict


class MultiPartyCall(PlivoResource):
    _name = 'MultiPartyCall'
    _identifier_string = 'mpc_uuid'

    def add_participant(self,
                        role,
                        from_=None,
                        to_=None,
                        call_uuid=None,
                        caller_name=None,
                        call_status_callback_url=None,
                        call_status_callback_method='POST',
                        sip_headers=None,
                        confirm_key=None,
                        confirm_key_sound_url=None,
                        confirm_key_sound_method='GET',
                        dial_music='Real',
                        ring_timeout=45,
                        delay_dial=0,
                        max_duration=14400,
                        max_participants=10,
                        record_min_member_count=1,
                        wait_music_url=None,
                        wait_music_method='GET',
                        agent_hold_music_url=None,
                        agent_hold_music_method='GET',
                        customer_hold_music_url=None,
                        customer_hold_music_method='GET',
                        recording_callback_url=None,
                        recording_callback_method='GET',
                        status_callback_url=None,
                        status_callback_method='GET',
                        on_exit_action_url=None,
                        on_exit_action_method='POST',
                        record=False,
                        record_participant_track=False,
                        record_file_format='mp3',
                        status_callback_events='mpc-state-changes,participant-state-changes',
                        stay_alone=False,
                        coach_mode=True,
                        mute=False,
                        hold=False,
                        start_mpc_on_enter=True,
                        end_mpc_on_exit=False,
                        relay_dtmf_inputs=False,
                        enter_sound='beep:1',
                        enter_sound_method='GET',
                        exit_sound='beep:2',
                        exit_sound_method='GET',
                        start_recording_audio=None,
                        start_recording_audio_method='GET',
                        stop_recording_audio=None,
                        stop_recording_audio_method='GET',
                        create_mpc_with_single_participant=True,
                        send_digits=None,
                        send_on_preanswer=False,
                        transcription_url=None,
                        transcript=False
                        ):
        return self.client.multi_party_calls.add_participant(role, uuid=self.id,
                                                             **to_param_dict(self.add_participant, locals()))

    def start(self):
        return self.client.multi_party_calls.start(uuid=self.id)

    def stop(self):
        return self.client.multi_party_calls.stop(uuid=self.id)

    def start_recording(self, file_format=None, recording_callback_url=None, recording_callback_method=None,
                        transcription_url=None, transcript=False):
        return self.client.multi_party_calls.start_recording(uuid=self.id,
                                                             **to_param_dict(self.add_participant, locals()))

    def stop_recording(self):
        return self.client.multi_party_calls.stop_recording(uuid=self.id)

    def pause_recording(self):
        return self.client.multi_party_calls.pause_recording(uuid=self.id)

    def resume_recording(self):
        return self.client.multi_party_calls.resume_recording(uuid=self.id)

    def start_participant_recording(self, participant_id, file_format=None, recording_callback_url=None,
                                    recording_callback_method=None, record_track_type='all',
                                    transcription_url=None, transcript=False):
        return self.client.multi_party_calls.start_participant_recording(participant_id=participant_id, uuid=self.id,
                                                                         **to_param_dict(
                                                                             self.start_participant_recording,
                                                                             locals()))

    def stop_participant_recording(self, participant_id, record_track_type='all'):
        return self.client.multi_party_calls.stop_participant_recording(participant_id=participant_id, uuid=self.id,
                                                                        **to_param_dict(
                                                                            self.stop_participant_recording,
                                                                            locals()))

    def pause_participant_recording(self, participant_id, record_track_type='all'):
        return self.client.multi_party_calls.pause_participant_recording(participant_id=participant_id, uuid=self.id,
                                                                         **to_param_dict(
                                                                             self.pause_participant_recording,
                                                                             locals()))

    def resume_participant_recording(self, participant_id, record_track_type='all'):
        return self.client.multi_party_calls.resume_participant_recording(participant_id=participant_id, uuid=self.id,
                                                                          **to_param_dict(
                                                                              self.resume_participant_recording,
                                                                              locals()))

    def get(self):
        return self.client.multi_party_calls.get(uuid=self.id)

    def update(self, participant_id, coach_mode=None, hold=None, mute=None):
        return self.client.multi_party_calls.update_participant(participant_id=participant_id,
                                                                uuid=self.id,
                                                                coach_mode=coach_mode,
                                                                mute=mute,
                                                                hold=hold)

    def start_play_audio(self, participant_id, url=None):
        return self.client.multi_party_calls.start_play_audio(participant_id=participant_id,
                                                              uuid=self.id,
                                                              url=url)

    def stop_play_audio(self, participant_id):
        return self.client.multi_party_calls.stop_play_audio(participant_id=participant_id, uuid=self.id)

    def kick(self, participant_id):
        return self.client.multi_party_calls.kick_participant(participant_id=participant_id, uuid=self.id)

    def get_participant(self, participant_id):
        return self.client.multi_party_calls.get_participant(participant_id=participant_id, uuid=self.id)


class MultiPartyCallParticipant(SecondaryPlivoResource):
    _name = 'MultiPartyCall'
    _identifier_string = 'mpc_uuid'
    _secondary_identifier_string = 'member_id'

    def start_participant_recording(self, file_format=None, recording_callback_url=None,
                                    recording_callback_method=None, record_track_type='all',
                                    transcription_url=None, transcript=False):
        return self.client.multi_party_calls.start_participant_recording(participant_id=self.secondary_id, uuid=self.id,
                                                                         **to_param_dict(
                                                                             self.start_participant_recording,
                                                                             locals()))

    def stop_participant_recording(self, record_track_type='all'):
        return self.client.multi_party_calls.stop_participant_recording(participant_id=self.secondary_id, uuid=self.id,
                                                                        **to_param_dict(self.stop_participant_recording,
                                                                                        locals()))

    def pause_participant_recording(self, record_track_type='all'):
        return self.client.multi_party_calls.pause_participant_recording(participant_id=self.secondary_id, uuid=self.id,
                                                                         **to_param_dict(
                                                                             self.pause_participant_recording,
                                                                             locals()))

    def resume_participant_recording(self, record_track_type='all'):
        return self.client.multi_party_calls.resume_participant_recording(participant_id=self.secondary_id,
                                                                          uuid=self.id,
                                                                          **to_param_dict(
                                                                              self.resume_participant_recording,
                                                                              locals()))

    def update(self, coach_mode=None, hold=None, mute=None):
        return self.client.multi_party_calls.update_participant(participant_id=self.secondary_id,
                                                                uuid=self.id,
                                                                coach_mode=coach_mode,
                                                                mute=mute,
                                                                hold=hold)

    def kick(self):
        return self.client.multi_party_calls.kick_participant(participant_id=self.secondary_id, uuid=self.id)

    def get(self):
        return self.client.multi_party_calls.get_participant(participant_id=self.secondary_id, uuid=self.id)


class MultiPartyCalls(PlivoResourceInterface):
    _resource_type = MultiPartyCall

    @staticmethod
    def __make_mpc_id(friendly_name=None, uuid=None):
        if not uuid and not friendly_name:
            raise ValidationError('specify either multi party call friendly name or uuid')
        if uuid and friendly_name:
            raise ValidationError('cannot specify both multi party call friendly name or uuid')
        return 'uuid_' + str(uuid) if uuid else 'name_' + str(friendly_name)

    @staticmethod
    def __clean_identifiers(params):
        params.pop('friendly_name', None)
        params.pop('uuid', None)
        params.pop('participant_id', None)
        return params

    @validate_args(
        sub_account=[optional(is_subaccount())],
        friendly_name=[optional(of_type_exact(str))],
        status=[optional(of_type_exact(str),
                         is_in(('active', 'initialized', 'ended'), case_sensitive=False, case_type='lower'))],
        termination_cause_code=[optional(of_type_exact(int))],
        end_time__gt=[optional(is_proper_date_format())],
        end_time__gte=[optional(is_proper_date_format())],
        end_time__lt=[optional(is_proper_date_format())],
        end_time__lte=[optional(is_proper_date_format())],
        creation_time__gt=[optional(is_proper_date_format())],
        creation_time__gte=[optional(is_proper_date_format())],
        creation_time__lt=[optional(is_proper_date_format())],
        creation_time__lte=[optional(is_proper_date_format())],
        limit=[optional(of_type(int), check(lambda limit: 0 < limit <= 20, '0 < limit <= 20'))],
        offset=[optional(of_type(int), check(lambda offset: 0 <= offset, '0 <= offset'))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def list(self,
             sub_account=None,
             friendly_name=None,
             status=None,
             termination_cause_code=None,
             end_time__gt=None,
             end_time__gte=None,
             end_time__lt=None,
             end_time__lte=None,
             creation_time__gt=None,
             creation_time__gte=None,
             creation_time__lt=None,
             creation_time__lte=None,
             limit=None,
             offset=None,
             callback_url=None,
             callback_method=None
             ):

        if not callback_url:
            return self.client.request('GET', ('MultiPartyCall',), to_param_dict(self.list, locals()),
                                       response_type=ListResponseObject, objects_type=MultiPartyCall,
                                       is_voice_request=True)
        else:
            return self.client.request('GET', ('MultiPartyCall',), to_param_dict(self.list, locals()),
                                       objects_type=MultiPartyCall, is_voice_request=True)

    @validate_args(
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def get(self, uuid=None, friendly_name=None, callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)

        if not callback_url:
            return self.client.request('GET', ('MultiPartyCall', mpc_id),
                                       is_voice_request=True, response_type=MultiPartyCall)
        else:
            return self.client.request('GET', ('MultiPartyCall', mpc_id),
                                       to_param_dict(self.get, locals()),
                                       is_voice_request=True, response_type=MultiPartyCall)

    @validate_args(
        role=[of_type_exact(str), is_in(('ai-agent', 'agent', 'supervisor', 'customer'), case_sensitive=False, case_type='lower')],
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        from_=[optional(is_phonenumber())],
        to_=[optional(is_iterable(of_type_exact(str), sep='<'))],
        call_uuid=[optional(of_type_exact(str))],
        caller_name=[optional(
            of_type_exact(str),
            check(lambda caller_name: 1 <= len(caller_name) <= 50, '1 < len(caller_name) <= 50'))],
        call_status_callback_url=[optional(of_type_exact(str), is_url())],
        call_status_callback_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        confirm_key=[
            optional(of_type_exact(str), is_in(('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '#', '*'),
                                               case_sensitive=False))],
        confirm_key_sound_url=[optional(of_type_exact(str), is_url())],
        confirm_key_sound_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        dial_music=[optional(
            of_type_exact(str),
            one_of(is_url(), is_in(('real', 'none'), case_sensitive=False, case_type='lower')),
        )],
        ring_timeout=[optional(one_of(of_type_exact(str), of_type_exact(int)), multiple_valid_integers())],
        delay_dial=[optional(one_of(of_type_exact(str), of_type_exact(int)), multiple_valid_integers())],
        max_duration=[optional(
            of_type_exact(int),
            check(lambda max_duration: 300 <= max_duration <= 28800, '300 < max_duration <= 28800'))],
        max_participants=[optional(
            of_type_exact(int),
            check(lambda max_participants: 2 <= max_participants <= 10, '2 < max_participants <= 10')
        )],
        record_min_member_count=[optional(
            of_type_exact(int),
            check(lambda record_min_member_count: 1 <= record_min_member_count <= 2, '1 < record_min_member_count <= 2')
        )],
        wait_music_url=[optional(of_type_exact(str), is_url())],
        wait_music_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        agent_hold_music_url=[optional(of_type_exact(str), is_url())],
        agent_hold_music_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        customer_hold_music_url=[optional(of_type_exact(str), is_url())],
        customer_hold_music_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        recording_callback_url=[optional(of_type_exact(str), is_url())],
        recording_callback_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        status_callback_url=[optional(of_type_exact(str), is_url())],
        status_callback_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        on_exit_action_url=[optional(of_type_exact(str), is_url())],
        on_exit_action_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        record=[optional(of_type_exact(bool))],
        record_participant_track=[optional(of_type_exact(bool))],
        record_file_format=[optional(of_type_exact(str), is_in(('mp3', 'wav'), case_sensitive=False,
                                                               case_type='lower'))],
        status_callback_events=[optional(of_type_exact(str), multi_is_in(('mpc-state-changes',
                                                                          'participant-state-changes',
                                                                          'participant-speak-events',
                                                                          'participant-digit-input-events',
                                                                          'add-participant-api-events'),
                                                                         case_sensitive=False,
                                                                         make_lower_case=True))],
        stay_alone=[optional(of_type_exact(bool))],
        coach_mode=[optional(of_type_exact(bool))],
        mute=[optional(of_type_exact(bool))],
        hold=[optional(of_type_exact(bool))],
        start_mpc_on_enter=[optional(of_type_exact(bool))],
        end_mpc_on_exit=[optional(of_type_exact(bool))],
        relay_dtmf_inputs=[optional(of_type_exact(bool))],
        enter_sound=[optional(
            of_type_exact(str),
            one_of(is_url(), is_in(('beep:1', 'beep:2', 'none'), case_sensitive=False, case_type='lower'))
        )],
        enter_sound_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        exit_sound=[optional(
            of_type_exact(str),
            one_of(is_url(), is_in(('beep:1', 'beep:2', 'none'), case_sensitive=False, case_type='lower')),
        )],
        exit_sound_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        start_recording_audio=[optional(of_type_exact(str), is_url())],
        start_recording_audio_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        stop_recording_audio=[optional(of_type_exact(str), is_url())],
        stop_recording_audio_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
        create_mpc_with_single_participant=[optional(of_type_exact(bool))],
        send_on_preanswer=[optional(of_type_exact(bool))],
    )
    def add_participant(self,
                        role,
                        friendly_name=None,
                        uuid=None,
                        from_=None,
                        to_=None,
                        call_uuid=None,
                        caller_name=None,
                        call_status_callback_url=None,
                        call_status_callback_method='POST',
                        sip_headers=None,
                        confirm_key=None,
                        confirm_key_sound_url=None,
                        confirm_key_sound_method='GET',
                        dial_music='Real',
                        ring_timeout=45,
                        delay_dial=0,
                        max_duration=14400,
                        max_participants=10,
                        record_min_member_count=1,
                        wait_music_url=None,
                        wait_music_method='GET',
                        agent_hold_music_url=None,
                        agent_hold_music_method='GET',
                        customer_hold_music_url=None,
                        customer_hold_music_method='GET',
                        recording_callback_url=None,
                        recording_callback_method='GET',
                        status_callback_url=None,
                        status_callback_method='GET',
                        on_exit_action_url=None,
                        on_exit_action_method='POST',
                        record=False,
                        record_participant_track=False,
                        record_file_format='mp3',
                        status_callback_events='mpc-state-changes,participant-state-changes',
                        stay_alone=False,
                        coach_mode=True,
                        mute=False,
                        hold=False,
                        start_mpc_on_enter=True,
                        end_mpc_on_exit=False,
                        relay_dtmf_inputs=False,
                        enter_sound='beep:1',
                        enter_sound_method='GET',
                        exit_sound='beep:2',
                        exit_sound_method='GET',
                        start_recording_audio=None,
                        start_recording_audio_method='GET',
                        stop_recording_audio=None,
                        stop_recording_audio_method='GET',
                        callback_url=None,
                        callback_method=None,
                        create_mpc_with_single_participant=True,
                        send_digits=None,
                        send_on_preanswer=False,
                        transcription_url=None,
                        transcript=False
                        ):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        caller_name = caller_name or from_
        if (from_ or to_) and call_uuid:
            raise ValidationError('cannot specify call_uuid when (from, to) is provided')
        if not from_ and not to_ and not call_uuid:
            raise ValidationError('specify either call_uuid or (from, to)')
        if call_uuid is None and (not from_ or not to_):
            raise ValidationError('specify (from, to) when not adding an existing call_uuid to multi party participant')
        if to_ and len(to_.split('<')) > 1 and role.lower() != "agent":
            raise ValidationError('Multiple to_  values given for role ' + role)
        elif to_ and len(to_.split('<')) > 20:
            raise ValidationError('No of to_ values provided should be lesser than 20')
        if to_ and len(str(ring_timeout).split('<')) > len(to_.split('<')):
            raise ValidationError("RingTimeout:number of ring_timeout(s) should be same as number of destination(s)")
        if to_ and len(str(delay_dial).split('<')) > len(to_.split('<')):
            raise ValidationError("DelayDial:number of delay_dial(s) should be same as number of destination(s)")
        return self.client.request('POST', ('MultiPartyCall', mpc_id, 'Participant'),
                                   self.__clean_identifiers(to_param_dict(self.add_participant, locals())),
                                   is_voice_request=True)

    @validate_args(
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def start(self, uuid=None, friendly_name=None, callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        localObject = {}
        localObject['status'] = 'active',
        if callback_url:
            localObject['callback_url'] = callback_url,
        if callback_method:
            localObject['callback_method'] = callback_method,
        return self.client.request('POST', ('MultiPartyCall', mpc_id), localObject)

    @validate_args(
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def stop(self, uuid=None, friendly_name=None, callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('DELETE', ('MultiPartyCall', mpc_id),
                                   to_param_dict(self.stop, locals()), is_voice_request=True)

    @validate_args(
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        file_format=[optional(of_type_exact(str), is_in(('mp3', 'wav'), case_sensitive=False,
                                                        case_type='lower'))],
        recording_callback_url=[optional(of_type_exact(str), is_url())],
        recording_callback_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def start_recording(self,
                        uuid=None,
                        friendly_name=None,
                        file_format='mp3',
                        recording_callback_url=None,
                        recording_callback_method='POST',
                        callback_url=None,
                        callback_method=None,
                        transcription_url=None,
                        transcript=False
                        ):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('POST', ('MultiPartyCall', mpc_id, 'Record'),
                                   self.__clean_identifiers(to_param_dict(self.start_recording, locals())),
                                   is_voice_request=True)

    @validate_args(
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def stop_recording(self, uuid=None, friendly_name=None, callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('DELETE', ('MultiPartyCall', mpc_id, 'Record'),
                                   to_param_dict(self.stop_recording, locals()), is_voice_request=True)

    @validate_args(
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def pause_recording(self, uuid=None, friendly_name=None, callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('POST', ('MultiPartyCall', mpc_id, 'Record', 'Pause'),
                                   to_param_dict(self.pause_recording, locals()), is_voice_request=True)

    @validate_args(
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def resume_recording(self, uuid=None, friendly_name=None, callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('POST', ('MultiPartyCall', mpc_id, 'Record', 'Resume'),
                                   to_param_dict(self.pause_recording, locals()), is_voice_request=True)

    @validate_args(
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        call_uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def list_participants(self, uuid=None, friendly_name=None, call_uuid=None, callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        if not callback_url:
            return self.client.request('GET', ('MultiPartyCall', mpc_id, 'Participant'),
                                       self.__clean_identifiers(to_param_dict(self.list_participants, locals())),
                                       response_type=ListResponseObject, objects_type=MultiPartyCallParticipant,
                                       is_voice_request=True)
        else:
            return self.client.request('GET', ('MultiPartyCall', mpc_id, 'Participant'),
                                       self.__clean_identifiers(to_param_dict(self.list_participants, locals())),
                                       objects_type=MultiPartyCallParticipant,
                                       is_voice_request=True)

    @validate_args(
        participant_id=[one_of(of_type_exact(str), of_type_exact(int))],
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        coach_mode=[optional(of_type_exact(bool))],
        mute=[optional(of_type_exact(bool))],
        hold=[optional(of_type_exact(bool))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def update_participant(self, participant_id, uuid=None, friendly_name=None, coach_mode=None, mute=None, hold=None,
                           callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        if str(participant_id).lower() == 'all' and coach_mode is not None:
            raise ValidationError('cannot specify coach_mode when updating all participants')
        if all(u is None for u in [coach_mode, mute, hold]):
            raise ValidationError('update at least one of coach_mode, mute or hold')
        return self.client.request('POST', ('MultiPartyCall', mpc_id, 'Participant', participant_id),
                                   self.__clean_identifiers(to_param_dict(self.update_participant, locals())),
                                   is_voice_request=True)

    @validate_args(
        participant_id=[one_of(of_type_exact(str), of_type_exact(int))],
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def kick_participant(self, participant_id, uuid=None, friendly_name=None, callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('DELETE', ('MultiPartyCall', mpc_id, 'Participant', participant_id),
                                   to_param_dict(self.kick_participant, locals()), is_voice_request=True)

    @validate_args(
        participant_id=[one_of(of_type_exact(str), of_type_exact(int))],
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def get_participant(self, participant_id, uuid=None, friendly_name=None,
                        callback_url=None, callback_method=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        if not callback_url:
            return self.client.request('GET', ('MultiPartyCall', mpc_id, 'Participant', participant_id),
                                       response_type=MultiPartyCallParticipant, is_voice_request=True)
        else:
            return self.client.request('GET', ('MultiPartyCall', mpc_id, 'Participant', participant_id),
                                       to_param_dict(self.get_participant, locals()), is_voice_request=True)

    @validate_args(
        participant_id=[one_of(of_type_exact(str), of_type_exact(int))],
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        file_format=[optional(of_type_exact(str), is_in(('mp3', 'wav'), case_sensitive=False,
                                                        case_type='lower'))],
        record_track_type=[optional(of_type_exact(str))],
        recording_callback_url=[optional(of_type_exact(str), is_url())],
        recording_callback_method=[optional(of_type_exact(str), is_in(('GET', 'POST'), case_sensitive=False))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def start_participant_recording(self, participant_id, uuid=None, friendly_name=None, file_format='mp3',
                                    recording_callback_url=None, recording_callback_method='POST',
                                    callback_url=None, callback_method=None, record_track_type='all',
                                    transcription_url=None, transcript=False):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('POST', ('MultiPartyCall', mpc_id, 'Participant', participant_id, 'Record'),
                                   self.__clean_identifiers(to_param_dict(self.start_participant_recording, locals())),
                                   is_voice_request=True)

    @validate_args(
        participant_id=[one_of(of_type_exact(str), of_type_exact(int))],
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
        record_track_type=[optional(of_type_exact(str))]
    )
    def stop_participant_recording(self,
                                   participant_id,
                                   uuid=None,
                                   friendly_name=None,
                                   callback_url=None,
                                   callback_method=None,
                                   record_track_type='all'):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('DELETE', ('MultiPartyCall', mpc_id, 'Participant', participant_id, 'Record'),
                                   to_param_dict(self.stop_participant_recording, locals()), is_voice_request=True)

    @validate_args(
        participant_id=[one_of(of_type_exact(str), of_type_exact(int))],
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
        record_track_type=[optional(of_type_exact(str))]
    )
    def pause_participant_recording(self, participant_id, uuid=None, friendly_name=None,
                                    callback_url=None, callback_method=None, record_track_type='all'):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('POST', ('MultiPartyCall', mpc_id, 'Participant', participant_id, 'Record', 'Pause'),
                                   to_param_dict(self.pause_participant_recording, locals()), is_voice_request=True)

    @validate_args(
        participant_id=[one_of(of_type_exact(str), of_type_exact(int))],
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
        record_track_type=[optional(of_type_exact(str))]
    )
    def resume_participant_recording(self, participant_id, uuid=None, friendly_name=None,
                                     callback_url=None, callback_method=None, record_track_type='all'):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('POST',
                                   ('MultiPartyCall', mpc_id, 'Participant', participant_id, 'Record', 'Resume'),
                                   to_param_dict(self.resume_participant_recording, locals()), is_voice_request=True)

    @validate_args(
        participant_id=[one_of(of_type_exact(str), of_type_exact(int))],
        friendly_name=[optional(of_type_exact(str))],
        uuid=[optional(of_type_exact(str))],
        url=[of_type_exact(str), is_url()]
    )
    def start_play_audio(self, participant_id, uuid=None, friendly_name=None, url=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('POST', ('MultiPartyCall', mpc_id, 'Member', participant_id, 'Play'),
                                   to_param_dict(self.start_play_audio, locals()), is_voice_request=True)

    def stop_play_audio(self, participant_id, uuid=None, friendly_name=None):
        mpc_id = self.__make_mpc_id(friendly_name, uuid)
        return self.client.request('DELETE', ('MultiPartyCall', mpc_id, 'Member', participant_id, 'Play'),
                                   is_voice_request=True)



from plivo.utils.validators import *
from plivo.xml import PlivoXMLElement, map_type


class RecordElement(PlivoXMLElement):
    _name = 'Record'
    _nestable = []

    @property
    def action(self):
        return self.__action

    @action.setter
    def action(self, value):
        self.__action = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_action(self, value):
        self.action = value
        return self

    @property
    def method(self):
        return self.__method

    @method.setter
    def method(self, value):
        self.__method = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_method(self, value):
        self.method = value
        return self

    @property
    def file_format(self):
        return self.__file_format

    @file_format.setter
    def file_format(self, value):
        self.__file_format = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_file_format(self, value):
        self.file_format = value
        return self

    @property
    def redirect(self):
        return self.__redirect

    @redirect.setter
    def redirect(self, value):
        self.__redirect = bool(value) if value is not None else None

    @validate_args(
        value=[of_type_exact(bool)],
    )
    def set_redirect(self, value):
        self.redirect = value
        return self

    @property
    def timeout(self):
        return self.__timeout

    @timeout.setter
    def timeout(self, value):
        self.__timeout = int(value) if value is not None else None

    @validate_args(
        value=[of_type(*six.integer_types)],
    )
    def set_timeout(self, value):
        self.timeout = value
        return self

    @property
    def max_length(self):
        return self.__max_length

    @max_length.setter
    def max_length(self, value):
        self.__max_length = int(value) if value is not None else None

    @validate_args(
        value=[of_type(*six.integer_types)],
    )
    def set_max_length(self, value):
        self.max_length = value
        return self

    @property
    def play_beep(self):
        return self.__play_beep

    @play_beep.setter
    def play_beep(self, value):
        self.__play_beep = bool(value) if value is not None else None

    @validate_args(
        value=[of_type_exact(bool)],
    )
    def set_play_beep(self, value):
        self.play_beep = value
        return self

    @property
    def finish_on_key(self):
        return self.__finish_on_key

    @finish_on_key.setter
    def finish_on_key(self, value):
        self.__finish_on_key = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_finish_on_key(self, value):
        self.finish_on_key = value
        return self

    @property
    def record_session(self):
        return self.__record_session

    @record_session.setter
    def record_session(self, value):
        self.__record_session = bool(value) if value is not None else None

    @validate_args(
        value=[of_type_exact(bool)],
    )
    def set_record_session(self, value):
        self.record_session = value
        return self

    @property
    def start_on_dial_answer(self):
        return self.__start_on_dial_answer

    @start_on_dial_answer.setter
    def start_on_dial_answer(self, value):
        self.__start_on_dial_answer = bool(
            value) if value is not None else None

    @validate_args(
        value=[of_type_exact(bool)],
    )
    def set_start_on_dial_answer(self, value):
        self.start_on_dial_answer = value
        return self

    @property
    def transcription_type(self):
        return self.__transcription_type

    @transcription_type.setter
    def transcription_type(self, value):
        self.__transcription_type = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_transcription_type(self, value):
        self.transcription_type = value
        return self

    @property
    def transcription_url(self):
        return self.__transcription_url

    @transcription_url.setter
    def transcription_url(self, value):
        self.__transcription_url = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_transcription_url(self, value):
        self.transcription_url = value
        return self

    @property
    def transcription_method(self):
        return self.__transcription_method

    @transcription_method.setter
    def transcription_method(self, value):
        self.__transcription_method = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_transcription_method(self, value):
        self.transcription_method = value
        return self
    
    @property
    def transcription_report_type(self):
        return self.__transcription_report_type

    @transcription_report_type.setter
    def transcription_report_type(self, value):
        self.__transcription_report_type = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_transcription_report_type(self, value):
        self.transcription_report_type = value
        return self

    @property
    def callback_url(self):
        return self.__callback_url

    @callback_url.setter
    def callback_url(self, value):
        self.__callback_url = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_callback_url(self, value):
        self.callback_url = value
        return self

    @property
    def callback_method(self):
        return self.__callback_method

    @callback_method.setter
    def callback_method(self, value):
        self.__callback_method = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_callback_method(self, value):
        self.callback_method = value
        return self

    @property
    def record_channel_type(self):
        return self.__record_channel_type

    @record_channel_type.setter
    def record_channel_type(self, value):
        self.__record_channel_type = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_record_channel_type(self, value):
        self.record_channel_type = value
        return self

    def __init__(
            self,
            action=None,
            method=None,
            file_format=None,
            redirect=None,
            timeout=None,
            max_length=None,
            play_beep=None,
            finish_on_key=None,
            record_session=None,
            start_on_dial_answer=None,
            transcription_type=None,
            transcription_url=None,
            transcription_method=None,
            transcription_report_type=None,
            callback_url=None,
            callback_method=None,
            record_channel_type=None
    ):
        super(RecordElement, self).__init__()

        self.action = action
        self.method = method
        self.file_format = file_format
        self.redirect = redirect
        self.timeout = timeout
        self.max_length = max_length
        self.play_beep = play_beep
        self.finish_on_key = finish_on_key
        self.record_session = record_session
        self.start_on_dial_answer = start_on_dial_answer
        self.transcription_type = transcription_type
        self.transcription_url = transcription_url
        self.transcription_report_type = transcription_report_type
        self.transcription_method = transcription_method
        self.callback_url = callback_url
        self.callback_method = callback_method
        self.record_channel_type = record_channel_type

    def to_dict(self):
        d = {
            'action': self.action,
            'method': self.method,
            'fileFormat': self.file_format,
            'redirect': self.redirect,
            'timeout': self.timeout,
            'maxLength': self.max_length,
            'playBeep': self.play_beep,
            'finishOnKey': self.finish_on_key,
            'recordSession': self.record_session,
            'startOnDialAnswer': self.start_on_dial_answer,
            'transcriptionType': self.transcription_type,
            'transcriptionUrl': self.transcription_url,
            'transcriptionMethod': self.transcription_method,
            'transcriptionReportType': self.transcription_report_type,
            'callbackUrl': self.callback_url,
            'callbackMethod': self.callback_method,
            'recordChannelType': self.record_channel_type
        }
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }



from ..utils.validators import *

from ..base import PlivoResource, PlivoResourceInterface, ListTollfreeVerificationResponseObject
from ..exceptions import *
from ..utils import *


class TollfreeVerification(PlivoResource):
    _name = 'TollfreeVerification'
    _identifier_string = 'uuid'

    def delete(self):
        return self.client.tollfree_verification.delete(tollfree_verification_uuid=self.uuid)

    def update(self,
               profile_uuid=None,
               usecase=None,
               usecase_summary=None,
               message_sample=None,
               optin_image_url=None,
               volume=None,
               additional_information=None,
               extra_data=None,
               optin_type=None,
               callback_url=None,
               callback_method=None):
        return self.client.tollfree_verification.update(self.uuid, profile_uuid=profile_uuid, usecase=usecase,
                                                        usecase_summary=usecase_summary, message_sample=message_sample,
                                                        optin_image_url=optin_image_url, volume=volume,
                                                        additional_information=additional_information,
                                                        extra_data=extra_data, optin_type=optin_type,
                                                        callback_url=callback_url, callback_method=callback_method)

class TollfreeVerifications(PlivoResourceInterface):
    def __init__(self, client):
        self._resource_type = TollfreeVerification
        super(TollfreeVerifications, self).__init__(client)

    def create(self,
               profile_uuid=None,
               usecase=None,
               usecase_summary=None,
               message_sample=None,
               optin_image_url=None,
               volume=None,
               number=None,
               additional_information=None,
               extra_data=None,
               optin_type=None,
               callback_url=None,
               callback_method=None):
        return self.client.request(
            'POST', ('TollfreeVerification',), to_param_dict(self.create, locals()))

    def get(self, tollfree_verification_uuid=None):
        return self.client.request(
            'GET', ('TollfreeVerification', tollfree_verification_uuid), response_type=TollfreeVerification)

    def list(self,
             number=None,
             status=None,
             profile_uuid=None,
             created__gt=None,
             created__gte=None,
             created__lt=None,
             created__lte=None,
             usecase=None,
             limit=20,
             offset=0):
        return self.client.request(
            'GET', ('TollfreeVerification',), to_param_dict(self.list, locals()),
            objects_type=TollfreeVerification, response_type=ListTollfreeVerificationResponseObject, )

    def update(self, tollfree_verification_uuid=None,
               profile_uuid=None,
               usecase=None,
               usecase_summary=None,
               message_sample=None,
               optin_image_url=None,
               volume=None,
               additional_information=None,
               extra_data=None,
               optin_type=None,
               callback_url=None,
               callback_method=None):
        return self.client.request(
            'POST', ('TollfreeVerification', tollfree_verification_uuid), to_param_dict(self.update, locals()))

    def delete(self, tollfree_verification_uuid=None):
        return self.client.request('DELETE', ('TollfreeVerification', tollfree_verification_uuid))
